@@ -6,7 +6,7 @@
  * functionallity, however they can't be implemented as backwards
  * compatibility procs.
  *---------------------------------------------------------------------------
- * Copyright 1991-1996 Karl Lehenbauer and Mark Diekhans.
+ * Copyright 1991-1997 Karl Lehenbauer and Mark Diekhans.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -15,7 +15,7 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclXunixSock.c,v 8.0 1996/11/21 00:25:18 markd Exp $
+ * $Id: tclXunixSock.c,v 8.4 1997/07/04 20:24:29 markd Exp $
  *-----------------------------------------------------------------------------
  */
 
@@ -115,7 +115,7 @@ BindFileHandles (interp, options, socketFD)
 
 
 /*-----------------------------------------------------------------------------
- * Tcl_ServerCreateCmd --
+ * TclX_ServerCreateCmd --
  *     Implements the TCL server_create command:
  *
  *        server_create ?options?
@@ -133,7 +133,7 @@ BindFileHandles (interp, options, socketFD)
  *-----------------------------------------------------------------------------
  */
 static int
-Tcl_ServerCreateCmd (clientData, interp, argc, argv)
+TclX_ServerCreateCmd (clientData, interp, argc, argv)
     ClientData  clientData;
     Tcl_Interp *interp;
     int         argc;
@@ -182,17 +182,18 @@ Tcl_ServerCreateCmd (clientData, interp, argc, argv)
         } else if (STREQU ("-reuseaddr", argv [nextArg])) {
             /* Ignore for compatibility */
         } else {
-            Tcl_AppendResult (interp, "expected ",
-                              "\"-myip\", \"-myport\", or \"-backlog\", ",
-                              "got \"", argv [nextArg], "\"", (char *) NULL);
+            TclX_AppendObjResult (interp, "expected ",
+                                  "\"-myip\", \"-myport\", or \"-backlog\", ",
+                                  "got \"", argv [nextArg], "\"",
+                                  (char *) NULL);
             return TCL_ERROR;
         }
         nextArg++;
     }
 
     if (nextArg != argc) {
-        Tcl_AppendResult (interp, tclXWrongArgs, argv[0],
-                          " ?options?", (char *) NULL);
+        TclX_AppendObjResult (interp, tclXWrongArgs, argv[0],
+                              " ?options?", (char *) NULL);
         return TCL_ERROR;
     }
 
@@ -228,25 +229,25 @@ Tcl_ServerCreateCmd (clientData, interp, argc, argv)
     channel = Tcl_MakeTcpClientChannel ((ClientData) socketFD);
     Tcl_RegisterChannel (interp, channel);
 
-    Tcl_AppendResult (interp, Tcl_GetChannelName (channel), (char *) NULL);
+    TclX_AppendObjResult (interp, Tcl_GetChannelName (channel), (char *) NULL);
     return TCL_OK;
 
     /*
      * Exit points for errors.
      */
   missingArg:
-    Tcl_AppendResult (interp, "missing argument for ", argv [nextArg],
-                      (char *) NULL);
+    TclX_AppendObjResult (interp, "missing argument for ", argv [nextArg],
+                          (char *) NULL);
     return TCL_ERROR;
 
   unixError:
-    interp->result = Tcl_PosixError (interp);
+    TclX_AppendObjResult (interp, Tcl_PosixError (interp), (char *) NULL);
     CloseForError (interp, channel, socketFD);
     return TCL_ERROR;
 }
 
 /*-----------------------------------------------------------------------------
- * Tcl_ServerAcceptCmd --
+ * TclX_ServerAcceptCmd --
  *     Implements the TCL server_accept command:
  *
  *        server_accept ?options? file
@@ -259,24 +260,26 @@ Tcl_ServerCreateCmd (clientData, interp, argc, argv)
  *-----------------------------------------------------------------------------
  */
 static int
-Tcl_ServerAcceptCmd (clientData, interp, argc, argv)
+TclX_ServerAcceptCmd (clientData, interp, argc, argv)
     ClientData  clientData;
     Tcl_Interp *interp;
     int         argc;
     char      **argv;
 {
-    Tcl_Channel channel;
-    Tcl_File acceptFile;
-    unsigned options;
-    int acceptSocketFD, addrLen;
-    int socketFD = -1, nextArg;
-    struct sockaddr_in connectSocket;
+    Tcl_Channel          channel;
+    unsigned             options;
+    int                  acceptSocketFD, addrLen;
+    int                  socketFD = -1;
+    int                  nextArg;
+    struct sockaddr_in   connectSocket;
 
     /*
      * Parse arguments.
      */
+
     nextArg = 1;
     options = SERVER_BUF;
+
     while ((nextArg < argc) && (argv [nextArg][0] == '-')) {
         if (STREQU ("-buf", argv [nextArg])) {
             options &= ~SERVER_NOBUF;
@@ -285,16 +288,17 @@ Tcl_ServerAcceptCmd (clientData, interp, argc, argv)
             options &= ~SERVER_BUF;
             options |= SERVER_NOBUF;
         } else {
-            Tcl_AppendResult (interp, "expected \"-buf\" or \"-nobuf\", ",
-                              "got \"", argv [nextArg], "\"", (char *) NULL);
+            TclX_AppendObjResult (interp, "expected \"-buf\" or \"-nobuf\", ",
+                                  "got \"", argv [nextArg], "\"",
+                                  (char *) NULL);
             return TCL_ERROR;
         }
         nextArg++;
     }
 
     if (nextArg != argc - 1) {
-        Tcl_AppendResult (interp, tclXWrongArgs, argv[0], " ?options? fileid",
-                          (char *) NULL);
+        TclX_AppendObjResult (interp, tclXWrongArgs, argv[0],
+                              " ?options? fileid", (char *) NULL);
         return TCL_ERROR;
     }
 
@@ -307,12 +311,18 @@ Tcl_ServerAcceptCmd (clientData, interp, argc, argv)
     if (channel == NULL)
         return TCL_ERROR;
 
-    acceptFile = Tcl_GetChannelFile (channel, TCL_READABLE);
-    if (acceptFile == NULL)
-        acceptFile = Tcl_GetChannelFile (channel, TCL_WRITABLE);
-    acceptSocketFD =  (int) Tcl_GetFileInfo (acceptFile, NULL);
+    if (Tcl_GetChannelHandle (channel, 
+			      TCL_READABLE, 
+			      (ClientData *)&acceptSocketFD) 
+	== TCL_ERROR) {
+        if (Tcl_GetChannelHandle (channel, 
+				  TCL_WRITABLE,
+				  (ClientData *)&acceptSocketFD) 
+	    == TCL_ERROR)
+	        return TCL_ERROR;
+    }
     if (acceptSocketFD < 0)
-        return TCL_ERROR;
+	return TCL_ERROR;
 
     addrLen = sizeof (connectSocket);
     socketFD = accept (acceptSocketFD, 
@@ -330,7 +340,7 @@ Tcl_ServerAcceptCmd (clientData, interp, argc, argv)
      * Exit points for errors.
      */
   unixError:
-    interp->result = Tcl_PosixError (interp);
+    TclX_AppendObjResult (interp, Tcl_PosixError (interp), (char *) NULL);
     if (socketFD >= 0)
         close (socketFD);
     return TCL_ERROR;
@@ -351,8 +361,10 @@ TclX_ServerInit (interp)
      * functionallity, however they can't be implemented as backwards
      * compatibility procs.
      */
-    Tcl_CreateCommand (interp, "server_accept", Tcl_ServerAcceptCmd,
+    Tcl_CreateCommand (interp, "server_accept", TclX_ServerAcceptCmd,
                        (ClientData) NULL, (void (*)()) NULL);
-    Tcl_CreateCommand (interp, "server_create", Tcl_ServerCreateCmd,
+    Tcl_CreateCommand (interp, "server_create", TclX_ServerCreateCmd,
                        (ClientData) NULL, (void (*)()) NULL);
 }
+
+
