@@ -8,7 +8,7 @@
  * is not available on that platform.  This results in code with minimal
  * number of #ifdefs.
  *-----------------------------------------------------------------------------
- * Copyright 1996-1996 Karl Lehenbauer and Mark Diekhans.
+ * Copyright 1996-1997 Karl Lehenbauer and Mark Diekhans.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -17,7 +17,7 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclXwinOS.c,v 8.0 1996/11/21 00:25:25 markd Exp $
+ * $Id: tclXwinOS.c,v 8.7 1997/08/08 16:51:14 markd Exp $
  *-----------------------------------------------------------------------------
  * The code for reading directories is based on TclMatchFiles from the Tcl
  * distribution file win/tclWinFile.c
@@ -27,6 +27,13 @@
 
 #include "tclExtdInt.h"
 
+typedef enum {
+    TCLX_WIN_CONSOLE,
+    TCLX_WIN_FILE,
+    TCLX_WIN_PIPE,
+    TCLX_WIN_SOCKET
+} tclXwinFileType;
+
 
 /*-----------------------------------------------------------------------------
  * TclXNotAvailableError --
@@ -43,10 +50,21 @@ int
 TclXNotAvailableError (Tcl_Interp *interp,
                        char       *funcName)
 {
-    Tcl_AppendResult (interp, funcName, " is not available on MS Windows",
-                      (char *) NULL);
+    TclX_AppendObjResult (interp, funcName, " is not available on MS Windows",
+                          (char *) NULL);
     return TCL_ERROR;
 }
+int
+TclXNotAvailableObjError (Tcl_Interp *interp,
+			  Tcl_Obj *obj)
+{
+    char *funcName = Tcl_GetStringFromObj(obj, NULL);
+
+    TclX_AppendObjResult (interp, funcName, " is not available on MS Windows",
+                          (char *) NULL);
+    return TCL_ERROR;
+}
+
 
 /*-----------------------------------------------------------------------------
  * TclX_SplitWinCmdLine --
@@ -60,65 +78,121 @@ TclXNotAvailableError (Tcl_Interp *interp,
  * Sun Microsystems, Inc.
  *-----------------------------------------------------------------------------
  */
+
+/*
+ *-------------------------------------------------------------------------
+ *
+ * setargv --
+ *
+ *	Parse the Windows command line string into argc/argv.  Done here
+ *	because we don't trust the builtin argument parser in crt0.  
+ *	Windows applications are responsible for breaking their command
+ *	line into arguments.
+ *
+ *	2N backslashes + quote -> N backslashes + begin quoted string
+ *	2N + 1 backslashes + quote -> literal
+ *	N backslashes + non-quote -> literal
+ *	quote + quote in a quoted string -> single quote
+ *	quote + quote not in quoted string -> empty string
+ *	quote -> begin quoted string
+ *
+ * Results:
+ *	Fills argcPtr with the number of arguments and argvPtr with the
+ *	array of arguments.
+ *
+ * Side effects:
+ *	Memory allocated.
+ *
+ *--------------------------------------------------------------------------
+ */
 void
 TclX_SplitWinCmdLine (int    *argcPtr,
                       char ***argvPtr)
 {
-    char *args = GetCommandLine();
-    char **argvlist, *p;
-    int size, i;
+    char *cmdLine, *p, *arg, *argSpace;
+    char **argv;
+    int argc, size, inquote, copy, slashes;
+    
+    cmdLine = GetCommandLine();
 
     /*
      * Precompute an overly pessimistic guess at the number of arguments
      * in the command line by counting non-space spans.
      */
-    for (size = 2, p = args; *p != '\0'; p++) {
-        if (isspace (*p)) {
-            size++;
-            while (isspace (*p)) {
-                p++;
-            }
-            if (*p == '\0') {
-                break;
-            }
-        }
-    }
-    argvlist = (char **) ckalloc ((unsigned) (size * sizeof (char *)));
-    *argvPtr = argvlist;
 
-    /*
-     * Parse the Windows command line string.  If an argument begins with a
-     * double quote, then spaces are considered part of the argument until the
-     * next double quote.  The argument terminates at the second quote.  Note
-     * that this is different from the usual Unix semantics.
-     */
-    for (i = 0, p = args; *p != '\0'; i++) {
-        while (isspace (*p)) {
-            p++;
-        }
-        if (*p == '\0') {
-            break;
-        }
-        if (*p == '"') {
-            p++;
-            (*argvPtr) [i] = p;
-            while ((*p != '\0') && (*p != '"')) {
-                p++;
-            }
-        } else {
-            (*argvPtr) [i] = p;
-            while (*p != '\0' && !isspace(*p)) {
-                p++;
-            }
-        }
-        if (*p != '\0') {
-            *p = '\0';
-            p++;
-        }
+    size = 2;
+    for (p = cmdLine; *p != '\0'; p++) {
+	if (isspace(*p)) {
+	    size++;
+	    while (isspace(*p)) {
+		p++;
+	    }
+	    if (*p == '\0') {
+		break;
+	    }
+	}
     }
-    (*argvPtr) [i] = NULL;
-    *argcPtr = i;
+    argSpace = (char *) ckalloc((unsigned) (size * sizeof(char *) 
+	    + strlen(cmdLine) + 1));
+    argv = (char **) argSpace;
+    argSpace += size * sizeof(char *);
+    size--;
+
+    p = cmdLine;
+    for (argc = 0; argc < size; argc++) {
+	argv[argc] = arg = argSpace;
+	while (isspace(*p)) {
+	    p++;
+	}
+	if (*p == '\0') {
+	    break;
+	}
+
+	inquote = 0;
+	slashes = 0;
+	while (1) {
+	    copy = 1;
+	    while (*p == '\\') {
+		slashes++;
+		p++;
+	    }
+	    if (*p == '"') {
+		if ((slashes & 1) == 0) {
+		    copy = 0;
+		    if ((inquote) && (p[1] == '"')) {
+			p++;
+			copy = 1;
+		    } else {
+			inquote = !inquote;
+		    }
+                }
+                slashes >>= 1;
+            }
+
+            while (slashes) {
+		*arg = '\\';
+		arg++;
+		slashes--;
+	    }
+
+	    if ((*p == '\0') || (!inquote && isspace(*p))) {
+		break;
+	    }
+	    if (copy != 0) {
+		*arg = *p;
+		arg++;
+	    }
+	    p++;
+        }
+	*arg = '\0';
+	argSpace = arg + 1;
+    }
+    argv[argc] = NULL;
+
+    *argcPtr = argc;
+    *argvPtr = argv;
 }
+
 
 /*-----------------------------------------------------------------------------
  * ChannelToHandle --
@@ -129,35 +203,61 @@ TclX_SplitWinCmdLine (int    *argcPtr,
  *   o channel - Channel to get file number for.
  *   o direction - TCL_READABLE or TCL_WRITABLE, or zero.  If zero, then
  *     return the first of the read and write numbers.
- *   o type - The type of the file. TCL_WIN_FILE if an error occurs
- *     (so something is in the value).  Maybe NULL.
+ *   o type - The type of the file. not set if an error occurs.
+ *
  * Returns:
- *   The file handle or NULL if a file number is not associated with this
- * access direction.  Just go ahead and pass this to a system call to
- * get a useful error message, they should never happen.  If type is 
- * TCL_WIN_SOCK, the return value is not valid.
+ *   The file handle or INVALID_HANDLE_VALUE if a HANDLE is not associated 
+ * with this access direction, or if the channel does not have a HANDLE
+ * of the Windows variety. We hope that the channel driver does not return
+ * a HANDLE that we cannot use.
  *-----------------------------------------------------------------------------
  */
-HANDLE
-ChannelToHandle (Tcl_Channel channel,
-                 int         direction,
-                 int        *type)
+static HANDLE
+ChannelToHandle (Tcl_Channel		channel,
+                 int         		direction,
+                 tclXwinFileType	*typePtr)
 {
-    Tcl_File file;
+    ClientData handle;
+    int	sockType;
+    int	sockTypeLen = sizeof(sockType);
     
     if (direction == 0) {
-        file = Tcl_GetChannelFile (channel, TCL_READABLE);
-        if (file == NULL)
-            file = Tcl_GetChannelFile (channel, TCL_WRITABLE);
+        if (Tcl_GetChannelHandle (channel, TCL_READABLE, &handle) != TCL_OK &&
+	    Tcl_GetChannelHandle (channel, TCL_WRITABLE, &handle) != TCL_OK) {
+	    handle = INVALID_HANDLE_VALUE;
+	}
     } else {
-        file = Tcl_GetChannelFile (channel, direction);
-        if (file == NULL) {
-            if (type != NULL)
-                *type = TCL_WIN_FILE;
-            return NULL;
-        }
+        if (Tcl_GetChannelHandle (channel, direction, &handle) != TCL_OK) {
+	    handle = INVALID_HANDLE_VALUE;
+	}
     }
-    return (HANDLE) Tcl_GetFileInfo (file, type);
+
+    /*
+     * Call GetFileType() even on invalid handles to set errno,
+     * also will coerce INVALID_SOCKET to INVALID_HANDLE,  they
+     * may not be the same on some machines.
+     */
+    switch (GetFileType ((HANDLE) handle)) {
+	case FILE_TYPE_DISK:
+	    *typePtr = TCLX_WIN_FILE;
+	    break;
+	case FILE_TYPE_CHAR:
+	    *typePtr = TCLX_WIN_CONSOLE;
+	    break;
+	case FILE_TYPE_PIPE:
+	    if (getsockopt ((SOCKET)handle, SOL_SOCKET, SO_TYPE, 
+			    (void *)&sockType, &sockTypeLen) == 0) {
+		*typePtr = TCLX_WIN_SOCKET;
+	    } else {
+		*typePtr = TCLX_WIN_PIPE;
+	    }
+	    break;
+	case FILE_TYPE_UNKNOWN:
+	    handle = INVALID_HANDLE_VALUE;
+	    break;
+    }
+    
+    return (HANDLE) handle;
 }
 
 /*-----------------------------------------------------------------------------
@@ -176,20 +276,19 @@ static SOCKET
 ChannelToSocket (Tcl_Interp  *interp,
                  Tcl_Channel  channel)
 {
-    Tcl_File file;
-    SOCKET sock;
-    int type;
-    
-    file = Tcl_GetChannelFile (channel, TCL_READABLE);
-    if (file == NULL)
-        file = Tcl_GetChannelFile (channel, TCL_WRITABLE);
-    sock = (SOCKET) Tcl_GetFileInfo (file, &type);
-    if (type != TCL_WIN_SOCKET) {
-        Tcl_AppendResult (interp, "channel \"", Tcl_GetChannelName (channel),
-                          "\" is not a socket", (char *) NULL);
+    ClientData handle;
+    tclXwinFileType type;
+
+    handle = ChannelToHandle(channel, 0, &type);
+
+    if (handle == INVALID_HANDLE_VALUE || type != TCLX_WIN_SOCKET) {
+        TclX_AppendObjResult (interp, "channel \"",
+                              Tcl_GetChannelName (channel),
+                              "\" is not a socket", (char *) NULL);
         return INVALID_SOCKET;
     }
-    return sock;
+
+    return (SOCKET) handle;
 }
 
 /*-----------------------------------------------------------------------------
@@ -280,18 +379,16 @@ TclXOSpipe (interp, channels)
 
     if (!CreatePipe (&readHandle, &writeHandle, &sec, 0)) {
 	TclWinConvertError (GetLastError ());
-        Tcl_AppendResult (interp, "pipe creation failed: ",
-                          Tcl_PosixError (interp), (char *) NULL);
+        TclX_AppendObjResult (interp, "pipe creation failed: ",
+                              Tcl_PosixError (interp), (char *) NULL);
         return TCL_ERROR;
     }
 
     channels [0] = Tcl_MakeFileChannel ((ClientData) readHandle,
-                                        (ClientData) -1,
                                         TCL_READABLE);
     Tcl_RegisterChannel (interp, channels [0]);
 
-    channels [1] = Tcl_MakeFileChannel ((ClientData) -1,
-                                        (ClientData) writeHandle,
+    channels [1] = Tcl_MakeFileChannel ((ClientData) writeHandle,
                                         TCL_WRITABLE);
     Tcl_RegisterChannel (interp, channels [1]);
 
@@ -368,7 +465,7 @@ TclXOSfsync (Tcl_Interp *interp,
     return TCL_OK;
 
   posixError:
-    Tcl_AppendResult (interp, Tcl_PosixError (interp), (char *) NULL);
+    TclX_AppendObjResult (interp, Tcl_PosixError (interp), (char *) NULL);
     return TCL_ERROR;
 }
 
@@ -403,8 +500,8 @@ TclXOSsystem (Tcl_Interp *interp,
                               NULL, NULL,
                               &si, &pi);
     if (!bSuccess) {
-        Tcl_AppendResult (interp, "process creation failed",
-                          (char *) NULL);
+        TclX_AppendObjResult (interp, "process creation failed",
+                              (char *) NULL);
         return TCL_ERROR;
     }
     CloseHandle (pi.hThread);
@@ -527,8 +624,6 @@ TclXOSkill (Tcl_Interp *interp,
  * Parameters:
  *   o interp - Errors are returned in result.
  *   o channel - Channel to get file number for.
- *   o direction - TCL_READABLE or TCL_WRITABLE, or zero.  If zero, then
- *     return the first of the read and write numbers.
  *   o statBuf - Status information, made to look as much like Unix as
  *     possible.
  *   o ttyDev - If not NULL, a boolean indicating if the device is
@@ -540,18 +635,25 @@ TclXOSkill (Tcl_Interp *interp,
 int
 TclXOSFstat (Tcl_Interp  *interp,
              Tcl_Channel  channel,
-             int          direction,
              struct stat *statBuf,
              int         *ttyDev)
 {
     HANDLE handle;
-    int type;
+    tclXwinFileType type;
     FILETIME creation, access, modify;
 
-    /*FIX: More of this information is availiable from
-     * GetFileInformationByHandle */
+    /* FIX: More of this information is availiable from
+     *      GetFileInformationByHandle
+     */
 
-    handle = ChannelToHandle (channel, direction, &type);
+    handle = ChannelToHandle (channel, 0, &type);
+
+    if (handle == INVALID_HANDLE_VALUE) {
+        TclX_AppendObjResult (interp, "channel \"",
+                              Tcl_GetChannelName (channel),
+                              "\" has no device handle", (char *) NULL);
+	return TCL_ERROR;
+    }
 
     /*
      * These don't translate to windows.
@@ -562,27 +664,27 @@ TclXOSFstat (Tcl_Interp  *interp,
 
     statBuf->st_mode = 0;
     switch (type) {
-      case TCL_WIN_PIPE:
+      case TCLX_WIN_PIPE:
         statBuf->st_mode |= S_IFIFO;
         break;
-      case TCL_WIN_FILE:
+      case TCLX_WIN_FILE:
         statBuf->st_mode |= S_IFREG;
         break;
-      case TCL_WIN_SOCKET:
+      case TCLX_WIN_SOCKET:
         statBuf->st_mode |= S_IFSOCK;
         break;
-      case TCL_WIN_CONSOLE:
+      case TCLX_WIN_CONSOLE:
         statBuf->st_mode |= S_IFCHR;
         break;
     }        
 
-    statBuf->st_nlink = (type == TCL_WIN_FILE) ? 1 : 0;
+    statBuf->st_nlink = (type == TCLX_WIN_FILE) ? 1 : 0;
     statBuf->st_uid = 0;   /* FIX??? */
     statBuf->st_gid = 0;
 
     switch (type) {
-      case TCL_WIN_FILE:
-      case TCL_WIN_PIPE:
+      case TCLX_WIN_FILE:
+      case TCLX_WIN_PIPE:
         statBuf->st_size = GetFileSize (handle, NULL);
         if (statBuf->st_size < 0)
             goto winError;
@@ -595,8 +697,8 @@ TclXOSFstat (Tcl_Interp  *interp,
         statBuf->st_ctime = ConvertToUnixTime (modify);
         break;
 
-      case TCL_WIN_SOCKET:
-      case TCL_WIN_CONSOLE:
+      case TCLX_WIN_SOCKET:
+      case TCLX_WIN_CONSOLE:
         statBuf->st_size = 0;
         statBuf->st_atime = 0;
         statBuf->st_mtime = 0;
@@ -605,57 +707,16 @@ TclXOSFstat (Tcl_Interp  *interp,
     }        
 
     if (ttyDev != NULL)
-        *ttyDev = (type == TCL_WIN_CONSOLE) ? 1 : 0;
+        *ttyDev = (type == TCLX_WIN_CONSOLE) ? 1 : 0;
     return TCL_OK;
 
   winError:
     TclWinConvertError (GetLastError ());
-    Tcl_AppendResult (interp, Tcl_PosixError (interp), (char *) NULL);
+    TclX_AppendObjResult (interp, Tcl_PosixError (interp), (char *) NULL);
     return TCL_ERROR;
 
 }
 
-/*-----------------------------------------------------------------------------
- * TclXOSSeakable --
- *   System dependent interface to determine if a channel is seekable.
- *
- * Parameters:
- *   o interp - Errors are returned in result.
- *   o channel - Channel to get the status of.
- *   o seekable - TRUE is return if seekable, FALSE if not.
- * Results:
- *   TCL_OK or TCL_ERROR.
- *-----------------------------------------------------------------------------
- */
-int
-TclXOSSeekable (interp, channel, seekablePtr)
-    Tcl_Interp  *interp;
-    Tcl_Channel  channel;
-    int         *seekablePtr;
-{
-    int type;
-
-    ChannelToHandle (channel, 0, &type);
-
-    switch (type) {
-      case TCL_WIN_PIPE:
-        *seekablePtr = FALSE;
-        break;
-      case TCL_WIN_FILE:
-        *seekablePtr = TRUE;
-        break;
-      case TCL_WIN_SOCKET:
-        *seekablePtr = FALSE;
-        break;
-      case TCL_WIN_CONSOLE:
-        *seekablePtr = FALSE;
-        break;
-      default:
-        panic ("TclXOSSeekable unknown file type %d", type);
-    }
-    return TCL_OK;
-}
-
 /*-----------------------------------------------------------------------------
  * TclXOSWalkDir --
  *   System dependent interface to reading the contents of a directory.  The
@@ -770,9 +831,10 @@ TclXOSWalkDir (Tcl_Interp       *interp,
         Tcl_DStringFree (&pathBuf);
         TclWinConvertError (GetLastError ());
         Tcl_ResetResult (interp);
-        Tcl_AppendResult (interp, "couldn't read volume information for \"",
-                          path, "\": ", Tcl_PosixError (interp),
-                          (char *) NULL);
+        TclX_AppendObjResult (interp,
+                              "couldn't read volume information for \"",
+                              path, "\": ", Tcl_PosixError (interp),
+                              (char *) NULL);
         return TCL_ERROR;
     }
 
@@ -791,9 +853,9 @@ TclXOSWalkDir (Tcl_Interp       *interp,
     if (handle == INVALID_HANDLE_VALUE) {
         TclWinConvertError (GetLastError ());
         Tcl_ResetResult (interp);
-        Tcl_AppendResult (interp, "couldn't read directory \"",
-                          path, "\": ", Tcl_PosixError (interp),
-                          (char *) NULL);
+        TclX_AppendObjResult (interp, "couldn't read directory \"",
+                              path, "\": ", Tcl_PosixError (interp),
+                              (char *) NULL);
         return TCL_ERROR;
     }
 
@@ -834,33 +896,34 @@ TclXOSWalkDir (Tcl_Interp       *interp,
  * Parameters:
  *   o channel - Channel.
  *   o fileSize - File size is returned here.
- *   o direction - TCL_READABLE or TCL_WRITABLE, or zero.  If zero, then
- *     return the first of the read and write numbers.
  * Results:
  *   TCL_OK or TCL_ERROR.  A POSIX error will be set.
  *-----------------------------------------------------------------------------
  */
 int
 TclXOSGetFileSize (Tcl_Channel  channel,
-                   int          direction,
                    off_t       *fileSize)
 {
     HANDLE handle; 
-    int type;
+    tclXwinFileType type;
 
-    handle = ChannelToHandle (channel, direction, &type);
+    handle = ChannelToHandle (channel, 0, &type);
+
+    if (handle == INVALID_HANDLE_VALUE) {
+	return TCL_ERROR;
+    }
     
     switch (type) {
-      case TCL_WIN_PIPE:
-      case TCL_WIN_FILE:
+      case TCLX_WIN_PIPE:
+      case TCLX_WIN_FILE:
         *fileSize = GetFileSize (handle, NULL);
         if (*fileSize < 0) {
             TclWinConvertError (GetLastError ());
             return TCL_ERROR;
         }
         break;
-      case TCL_WIN_SOCKET:
-      case TCL_WIN_CONSOLE:
+      case TCLX_WIN_SOCKET:
+      case TCLX_WIN_CONSOLE:
         *fileSize = 0;
     }
     return TCL_OK;
@@ -885,7 +948,64 @@ TclXOSftruncate (Tcl_Interp  *interp,
                  off_t        newSize,
                  char        *funcName)
 {
-    return TclXNotAvailableError (interp, funcName);
+    HANDLE handle;
+    int pos;
+    tclXwinFileType type;
+
+    handle = ChannelToHandle (channel, TCL_WRITABLE, &type);
+
+    if (handle == INVALID_HANDLE_VALUE) {
+        TclX_AppendObjResult (interp, "channel \"",
+                              Tcl_GetChannelName (channel),
+                              "\" was not open for write access",
+                              (char *) NULL);
+        return TCL_ERROR;
+    }
+    if (type != TCLX_WIN_FILE) {
+        TclX_AppendObjResult (interp, "truncation of \"",
+                              Tcl_GetChannelName (channel),
+                              "\" failed: can only truncate disk files",
+                              (char *) NULL);
+	return TCL_ERROR;
+    }
+    pos = Tcl_Tell (channel);
+    if (SetFilePointer (handle, (LONG)newSize, NULL,
+                        FILE_BEGIN) == 0xFFFFFFFF) {
+        TclWinConvertError (GetLastError ());
+        TclX_AppendObjResult (interp, "truncation of \"",
+                              Tcl_GetChannelName (channel),
+                              "\" failed: ", Tcl_PosixError (interp),
+                              (char *) NULL);
+        return TCL_ERROR;
+    }
+    /*
+     * FIX: we really ought to interpolate zeros when extending the file,
+     * since SetEndOfFile does not promise to do this.
+     */
+    if (!SetEndOfFile (handle)) {
+        TclWinConvertError (GetLastError ());
+        TclX_AppendObjResult (interp, "truncation of \"",
+                              Tcl_GetChannelName (channel),
+                              "\" failed: ", Tcl_PosixError (interp),
+                              (char *) NULL);
+        if (pos >= 0) {
+            (void) SetFilePointer (handle, (LONG)pos, NULL, FILE_BEGIN);
+        }
+        return TCL_ERROR;
+    }
+    if (pos >= 0) {
+        if (SetFilePointer (handle, (LONG)pos, NULL,
+                            FILE_BEGIN) == 0xFFFFFFFF) {
+            TclWinConvertError (GetLastError ());
+            TclX_AppendObjResult (interp, "couldn't restore position after ",
+                                  "truncating \"",
+                                  Tcl_GetChannelName (channel),
+                                  "\": ", Tcl_PosixError (interp),
+                                  (char *) NULL);
+            return TCL_ERROR;
+        }
+    }
+    return TCL_OK;
 }
 
 /*-----------------------------------------------------------------------------
@@ -902,9 +1022,9 @@ TclXOSftruncate (Tcl_Interp  *interp,
  */
 int
 TclXOSfork (Tcl_Interp *interp,
-            char       *funcName)
+            Tcl_Obj    *funcNameObj)
 {
-    return TclXNotAvailableError (interp, funcName);
+    return TclXNotAvailableObjError (interp, funcNameObj);
 }
 
 /*-----------------------------------------------------------------------------
@@ -930,8 +1050,8 @@ TclXOSexecl (Tcl_Interp *interp,
 
     pid = spawnvp (_P_NOWAIT , path, argList);
     if (pid == -1) {
-        Tcl_AppendResult (interp, "exec of \"", path, "\" failed: ",
-                          Tcl_PosixError (interp), (char *) NULL);
+        TclX_AppendObjResult (interp, "exec of \"", path, "\" failed: ",
+                              Tcl_PosixError (interp), (char *) NULL);
         return TCL_ERROR;
     }
 
@@ -963,8 +1083,8 @@ TclXOSInetAtoN (Tcl_Interp     *interp,
     if (inAddress->s_addr != INADDR_NONE)
         return TCL_OK;
     if (interp != NULL) {
-        Tcl_AppendResult (interp, "malformed address: \"",
-                          strAddress, "\"", (char *) NULL);
+        TclX_AppendObjResult (interp, "malformed address: \"",
+                              strAddress, "\"", (char *) NULL);
     }
     return TCL_ERROR;
 }
@@ -994,8 +1114,8 @@ TclXOSgetpeername (Tcl_Interp *interp,
     if (sock == INVALID_SOCKET)
         return TCL_ERROR;
     if (getpeername (sock, (struct sockaddr *) sockaddr, &sockaddrSize) < 0) {
-        Tcl_AppendResult (interp, Tcl_GetChannelName (channel), ": ",
-                          Tcl_PosixError (interp), (char *) NULL);
+        TclX_AppendObjResult (interp, Tcl_GetChannelName (channel), ": ",
+                              Tcl_PosixError (interp), (char *) NULL);
         return TCL_ERROR;
     }
     return TCL_OK;
@@ -1027,8 +1147,8 @@ TclXOSgetsockname (Tcl_Interp *interp,
         return TCL_ERROR;
 
     if (getsockname (sock, (struct sockaddr *) sockaddr, &sockaddrSize) < 0) {
-        Tcl_AppendResult (interp, Tcl_GetChannelName (channel), ": ",
-                          Tcl_PosixError (interp), (char *) NULL);
+        TclX_AppendObjResult (interp, Tcl_GetChannelName (channel), ": ",
+                              Tcl_PosixError (interp), (char *) NULL);
         return TCL_ERROR;
     }
     return TCL_OK;
@@ -1063,8 +1183,8 @@ TclXOSgetsockopt (interp, channel, option, valuePtr)
 
     if (getsockopt (sock, SOL_SOCKET, option, 
                     (void*) valuePtr, &valueLen) != 0) {
-        Tcl_AppendResult (interp, Tcl_GetChannelName (channel), ": ",
-                          Tcl_PosixError (interp), (char *) NULL);
+        TclX_AppendObjResult (interp, Tcl_GetChannelName (channel), ": ",
+                              Tcl_PosixError (interp), (char *) NULL);
         return TCL_ERROR;
     }
     return TCL_OK;
@@ -1099,8 +1219,8 @@ TclXOSsetsockopt (interp, channel, option, value)
 
     if (setsockopt (sock, SOL_SOCKET, option,
                     (void*) &value, valueLen) != 0) {
-        Tcl_AppendResult (interp, Tcl_GetChannelName (channel), ": ",
-                          Tcl_PosixError (interp), (char *) NULL);
+        TclX_AppendObjResult (interp, Tcl_GetChannelName (channel), ": ",
+                              Tcl_PosixError (interp), (char *) NULL);
         return TCL_ERROR;
     }
     return TCL_OK;
@@ -1127,13 +1247,13 @@ TclXOSchmod (interp, fileName, mode)
 #if 0
     /*FIX:*/
     if (chmod (fileName, (unsigned short) mode) < 0) {
-        Tcl_AppendResult (interp, "chmod failed on \"", fileName, "\": ",
-                          Tcl_PosixError (interp), (char *) NULL);
+        TclX_AppendObjResult (interp, "chmod failed on \"", fileName, "\": ",
+                              Tcl_PosixError (interp), (char *) NULL);
         return TCL_ERROR;
     }
     return TCL_OK;
 #else
-    Tcl_AppendResult (interp, "chmod is not available on this system",
+    TclX_AppendObjResult (interp, "chmod is not available on this system",
                       (char *) NULL);
     return TCL_ERROR;
 #endif
@@ -1162,12 +1282,12 @@ TclXOSfchmod (interp, channel, mode, funcName)
 #if 0
   FIX:
     if (fchmod (ChannelToFnum (channel, 0), (unsigned short) mode) < 0) {
-        Tcl_AppendResult (interp, Tcl_PosixError (interp), (char *) NULL);
+        TclX_AppendObjResult (interp, Tcl_PosixError (interp), (char *) NULL);
         return TCL_ERROR;
     }
 #else
-    Tcl_AppendResult (interp, funcName, " is not available on this system",
-                      (char *) NULL);
+    TclX_AppendObjResult (interp, funcName, " is not available on this system",
+                          (char *) NULL);
     return TCL_ERROR;
 #endif
 }
@@ -1193,13 +1313,14 @@ TclXOSfchmod (interp, channel, mode, funcName)
  *   TCL_OK or TCL_ERROR.
  *-----------------------------------------------------------------------------
  */
+
 int
-TclXOSChangeOwnGrp (interp, options, ownerStr, groupStr, files, funcName)
+TclXOSChangeOwnGrpObj (interp, options, ownerStr, groupStr, files, funcName)
     Tcl_Interp  *interp;
     unsigned     options;
     char        *ownerStr;
     char        *groupStr;
-    char       **files;
+    Tcl_Obj	*files;
     char       *funcName;
 {
     return TclXNotAvailableError (interp, funcName);
@@ -1227,12 +1348,12 @@ TclXOSChangeOwnGrp (interp, options, ownerStr, groupStr, files, funcName)
  *-----------------------------------------------------------------------------
  */
 int
-TclXOSFChangeOwnGrp (interp, options, ownerStr, groupStr, channelIds, funcName)
+TclXOSFChangeOwnGrpObj (interp, options, ownerStr, groupStr, channelIds, funcName)
     Tcl_Interp *interp;
     unsigned    options;
     char       *ownerStr;
     char       *groupStr;
-    char      **channelIds;
+    Tcl_Obj    *channelIds;
     char       *funcName;
 {
     return TclXNotAvailableError (interp, funcName);
@@ -1246,10 +1367,8 @@ TclXOSFChangeOwnGrp (interp, options, ownerStr, groupStr, channelIds, funcName)
  *   o interp - Pointer to the current interpreter, error messages will be
  *     returned in the result.
  *   o channel - Channel to get the numbers for.
- *   o readFnumPtr - The read file number is returned here.  -1 is returned if
- *     the file is not open for reading.
- *   o writeFnumPtr - The write file number is returned here.  -1 is returned
- *     if the file is not open for writing.
+ *   o direction - TCL_READABLE or TCL_WRITABLE.
+ *   o fnumPtr - The file number for the direction is returned here.
  * Returns:
  *   TCL_OK or TCL_ERROR.
  *-----------------------------------------------------------------------------
@@ -1257,35 +1376,31 @@ TclXOSFChangeOwnGrp (interp, options, ownerStr, groupStr, channelIds, funcName)
 int
 TclXOSGetSelectFnum (Tcl_Interp *interp,
                      Tcl_Channel channel,
-                     int        *readFnumPtr,
-                     int        *writeFnumPtr)
+                     int         direction,
+                     int        *fnumPtr)
 {
-    Tcl_File file;
-    int type;
+    tclXwinFileType type;
+    HANDLE handle = ChannelToHandle (channel, direction, &type);
+    
+    if (handle == INVALID_HANDLE_VALUE) {
+        TclX_AppendObjResult (interp,  "channel \"",
+                              Tcl_GetChannelName (channel),
+                              "\" was not open for requested access",
+                              (char *) NULL);
+        return TCL_ERROR;
+    }
 
-    file = Tcl_GetChannelFile (channel, TCL_READABLE);
-    if (file != NULL) {
-        *readFnumPtr = (int) ((SOCKET) Tcl_GetFileInfo (file, &type));
-        if (type != TCL_WIN_SOCKET)
-            goto badFileType;
-    } else {
-        *readFnumPtr = -1;
-    }
-    file = Tcl_GetChannelFile (channel, TCL_WRITABLE);
-    if (file != NULL) {
-        *writeFnumPtr = (int) ((SOCKET) Tcl_GetFileInfo (file, &type));
-        if (type != TCL_WIN_SOCKET)
-            goto badFileType;
-    } else {
-        *writeFnumPtr = -1;
+    if (type != TCLX_WIN_SOCKET) {
+        TclX_AppendObjResult (interp, "channel \"",
+			      Tcl_GetChannelName (channel),
+                              "\" is not a socket; select only works on ",
+                              "sockets on Windows", (char *) NULL);
+        return TCL_ERROR;
     }
+
+    *fnumPtr = (int) handle;
     return TCL_OK;
     
-  badFileType:
-    Tcl_AppendResult (interp, Tcl_GetChannelName (channel),
-                      " is not a socket; select only works on sockets on ",
-                      "Windows", (char *) NULL);
-    return TCL_ERROR;
 }
 
 /*-----------------------------------------------------------------------------
@@ -1340,24 +1455,33 @@ LockUnlockSetup (Tcl_Interp     *interp,
                  char           *whichMsg)
 {
     HANDLE handle;
-    int type;
+    tclXwinFileType type;
 
     /*
      * Get the handle and validate that this is something we can lock.
      */
     handle = ChannelToHandle (lockInfoPtr->channel, 0, &type);
+
+    if (handle == INVALID_HANDLE_VALUE) {
+        TclX_AppendObjResult (interp, "channel \"",
+                              Tcl_GetChannelName (lockInfoPtr->channel),
+                              "\" has no device handle", (char *) NULL);
+	return handle;
+    }
+
     switch (type) {
-      case TCL_WIN_PIPE:
-        Tcl_AppendResult (interp, "can't lock a pipe line under MS Windows",
-                          (char *) NULL);
-        return NULL;
-      case TCL_WIN_FILE:
+      case TCLX_WIN_PIPE:
+        TclX_AppendObjResult (interp,
+                              "can't lock a pipe line under MS Windows",
+                              (char *) NULL);
+        return INVALID_HANDLE_VALUE;
+      case TCLX_WIN_FILE:
         break;
-      case TCL_WIN_SOCKET:
-        Tcl_AppendResult (interp, "can't lock a socket under windows",
-                          (char *) NULL);
-        return NULL;
-      case TCL_WIN_CONSOLE:
+      case TCLX_WIN_SOCKET:
+        TclX_AppendObjResult (interp, "can't lock a socket under windows",
+                              (char *) NULL);
+        return INVALID_HANDLE_VALUE;
+      case TCLX_WIN_CONSOLE:
         break;  /* FIX: Is this legal?? */
       default:
         panic ("unknown win channel type %d\n", type);
@@ -1404,11 +1528,11 @@ LockUnlockSetup (Tcl_Interp     *interp,
   winError:
     TclWinConvertError (GetLastError ());
     lockInfoPtr->gotLock = FALSE;
-    Tcl_AppendResult (interp, whichMsg, " of \"",
-                      Tcl_GetChannelName (lockInfoPtr->channel),
-                      "\" failed: ", Tcl_PosixError (interp),
-                      (char *) NULL);
-    return NULL;
+    TclX_AppendObjResult (interp, whichMsg, " of \"",
+                          Tcl_GetChannelName (lockInfoPtr->channel),
+                          "\" failed: ", Tcl_PosixError (interp),
+                          (char *) NULL);
+    return INVALID_HANDLE_VALUE;
 }
 
 /*-----------------------------------------------------------------------------
@@ -1438,7 +1562,8 @@ TclXOSFlock (interp, lockInfoPtr)
                               &lengthLow,
                               &lengthHigh,
                               "lock");
-    if (handle == NULL)
+
+    if (handle == INVALID_HANDLE_VALUE)
         return TCL_ERROR;
 
     flags = 0;
@@ -1459,14 +1584,14 @@ TclXOSFlock (interp, lockInfoPtr)
 
   winError:
     if (GetLastError () == ERROR_CALL_NOT_IMPLEMENTED) {
-        Tcl_AppendResult (interp, "file locking is not yet available on ",
-                          "Windows 3.1 and 95", (char *) NULL);
+        TclX_AppendObjResult (interp, "file locking is not yet available on ",
+                              "Windows 3.1 and 95", (char *) NULL);
     } else {
         TclWinConvertError (GetLastError ());
-        Tcl_AppendResult (interp, "lock of \"",
-                          Tcl_GetChannelName (lockInfoPtr->channel),
-                          "\" failed: ", Tcl_PosixError (interp),
-                          (char *) NULL);
+        TclX_AppendObjResult (interp, "lock of \"",
+                              Tcl_GetChannelName (lockInfoPtr->channel),
+                              "\" failed: ", Tcl_PosixError (interp),
+                              (char *) NULL);
     }
     lockInfoPtr->gotLock = FALSE;
     return TCL_ERROR;
@@ -1499,7 +1624,7 @@ TclXOSFunlock (interp, lockInfoPtr)
                               &lengthLow,
                               &lengthHigh,
                               "unlock");
-    if (handle == NULL)
+    if (handle == INVALID_HANDLE_VALUE)
         return TCL_ERROR;
 
     if (!UnlockFileEx (handle, 0, lengthLow, lengthHigh, &start)) {
@@ -1509,14 +1634,14 @@ TclXOSFunlock (interp, lockInfoPtr)
 
   winError:
     if (GetLastError () == ERROR_CALL_NOT_IMPLEMENTED) {
-        Tcl_AppendResult (interp, "file locking is not yet available on ",
-                          "Windows 3.1 and 95", (char *) NULL);
+        TclX_AppendObjResult (interp, "file locking is not yet available on ",
+                              "Windows 3.1 and 95", (char *) NULL);
     } else {
         TclWinConvertError (GetLastError ());
-        Tcl_AppendResult (interp, "unlock of \"",
-                          Tcl_GetChannelName (lockInfoPtr->channel),
-                          "\" failed: ", Tcl_PosixError (interp),
-                          (char *) NULL);
+        TclX_AppendObjResult (interp, "unlock of \"",
+                              Tcl_GetChannelName (lockInfoPtr->channel),
+                              "\" failed: ", Tcl_PosixError (interp),
+                              (char *) NULL);
     }
     return TCL_ERROR;
 }
@@ -1586,9 +1711,36 @@ TclXOSGetCloseOnExec (interp, channel, valuePtr)
     Tcl_Channel channel;
     int        *valuePtr;
 {
-    /* FIX: Implement */
-    return TclXNotAvailableError (interp,
-                                  "close-on-exec mode");
+    HANDLE handle;
+    tclXwinFileType type;
+    DWORD flags;
+
+    handle = ChannelToHandle (channel, 0, &type);
+
+    if (handle == INVALID_HANDLE_VALUE) {
+        TclX_AppendObjResult (interp, "channel \"",
+                              Tcl_GetChannelName (channel),
+                              "\" has no device handle", (char *) NULL);
+	return TCL_ERROR;
+    }
+
+    /*
+     * The following works on Windows NT, but not on Windows 95.
+     */
+    if (!GetHandleInformation (handle, &flags)) {
+        TclWinConvertError (GetLastError ());
+        TclX_AppendObjResult (interp, "getting close-on-exec for \"",
+                              Tcl_GetChannelName (channel),
+                              "\" failed: ", Tcl_PosixError (interp),
+                              (char *) NULL);
+        return TCL_ERROR;
+    }
+
+    /*
+     * N.B. The value of the CLOEXEC flag is the inverse of HANDLE_FLAG_INHERIT.
+     */
+    *valuePtr = (flags & HANDLE_FLAG_INHERIT) ? 0 : 1;
+    return TCL_OK;
 }
 
 /*-----------------------------------------------------------------------------
@@ -1610,7 +1762,33 @@ TclXOSSetCloseOnExec (interp, channel, value)
     Tcl_Channel channel;
     int         value;
 {
-    /* FIX: Implement */
-    return TclXNotAvailableError (interp,
-                                  "close-on-exec mode");
+    HANDLE handle;
+    tclXwinFileType type;
+
+    handle = ChannelToHandle (channel, 0, &type);
+
+    if (handle == INVALID_HANDLE_VALUE) {
+        TclX_AppendObjResult (interp, "channel \"",
+                              Tcl_GetChannelName (channel),
+                              "\" has no device handle", (char *) NULL);
+	return TCL_ERROR;
+    }
+
+    /*
+     * The following works on Windows NT, but not on Windows 95.
+     * N.B. The value of the CLOEXEC flag is the inverse of HANDLE_FLAG_INHERIT.
+     */
+    if (!SetHandleInformation (handle,
+                               HANDLE_FLAG_INHERIT,
+                               value ? 0 : HANDLE_FLAG_INHERIT)) {
+        TclWinConvertError (GetLastError ());
+        TclX_AppendObjResult (interp, "setting close-on-exec for \"",
+                              Tcl_GetChannelName (channel),
+                              "\" failed: ", Tcl_PosixError (interp),
+                              (char *) NULL);
+        return TCL_ERROR;
+    }
+    return TCL_OK;
 }
+
+
